<?php

namespace Drupal\fluent\Plugin\FluentFieldItemResolver;

use Drupal\fluent\Plugin\FluentFieldItemResolverPluginBase;
use Drupal\options\Plugin\Field\FieldType\ListFloatItem;
use Drupal\options\Plugin\Field\FieldType\ListIntegerItem;
use Drupal\options\Plugin\Field\FieldType\ListItemBase;

/**
 * Plugin implementation of the fluent_field_item_resolver.
 *
 * @FluentFieldItemResolver(
 *   id = "list",
 *   label = @Translation("List field"),
 * )
 */
class ListItem extends FluentFieldItemResolverPluginBase {

  /**
   *
   */
  public function can($field): bool {
    return $field instanceof ListItemBase;
  }

  /**
   * @param \Drupal\options\Plugin\Field\FieldType\ListItemBase $field
   *
   * @return mixed|void
   */
  public function handler($field) {
    $key = $field->get('value')->getValue();

    return [$key => $this->castValue($field)];
  }

  /**
   * @param \Drupal\options\Plugin\Field\FieldType\ListItemBase $field
   *
   * @return float|int|string
   */
  protected function castValue($field) {
    $value = $field->get('value')->getValue();

    if ($field instanceof ListFloatItem) {
      return (float) $value;
    }
    elseif ($field instanceof ListIntegerItem) {
      return (int) $value;
    }

    return $value;
  }

}
