<?php

namespace Drupal\fluent\Plugin\FluentFieldResolver;

use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\fluent\Exception\NotFieldFoundException;
use Drupal\fluent\FieldControl;
use Drupal\fluent\Plugin\FluentFieldResolverBase;

/**
 * @FluentFieldResolver(
 *   id = "common_field",
 *   label = "Commond Field Resolver",
 *   weight = 100
 * )
 */
class CommonField extends FluentFieldResolverBase {

  /**
   *
   */
  public function handler($object, $fieldName, FieldControl $fieldControl) {
    if ($object instanceof FieldItemListInterface) {
      $cardinality = $object->getFieldDefinition()->getFieldStorageDefinition()->getCardinality();

      $items = [];
      foreach ($object as $item) {
        $items[] = $item;
      }

      if ($fieldControl->isMultivalued() && ($cardinality == -1 | $cardinality > 1)) {
        return $items;
      }

      return $object->first();
    }

    // @todo refactor this code.
    // First try to access as a field
    // if not exist try to access as a method
    $field = NULL;
    try {
      $field = $object->get($fieldName);
    }
    catch (\InvalidArgumentException $e) {
      if ($method = $this->getMethod($object, $fieldName)) {
        $field = $object->{$method}();
      }

      if ($field === NULL) {
        // @todo Error handling.
        $fields = array_keys($object->getFields());

        // Remove __ methods.
        $methods = get_class_methods($object);
        $methods = preg_replace('/__./', NULL, $methods);

        $message = sprintf(
          'Available fields: %s

      Available methods: %s',
          implode(', ', $fields),
          implode(', ', $methods)
        );

        throw new NotFieldFoundException($message);
      }
    }

    return $field;
  }

  /**
   * @param  \Drupal\Core\Field\FieldDefinitionInterface $fieldDefinition
   * @return bool
   */
  public function can(FieldDefinitionInterface $fieldDefinition): bool {
    return TRUE;
  }

  /**
   *
   */
  public function hasField($object, $fieldName) {
    return method_exists($object, 'hasField') && $object->hasField($fieldName);
  }

  /**
   *
   */
  public function getMethod($object, $possibleName) {
    if ($this->isMethod($object, $possibleName)) {
      return $possibleName;
    }

    $prefixes = [
      'get',
      'is',
      'to',
    ];

    foreach ($prefixes as $prefix) {
      $name = $prefix . ucfirst($possibleName);
      if ($this->isMethod($object, $name)) {
        return $name;
      }
    }

  }

  /**
   * @param $entity
   * @param $name
   *
   * @return bool
   */
  public function isMethod($entity, $name) {
    return method_exists($entity, $name);
  }

}
